use std::{collections::HashMap, error::Error, fmt::Display};

use crate::{
    handling::methods::Method,
    utils::{mime::Mime, urlencoded::UrlEncodeData},
};

type HeaderMap = Vec<String>;

/// A struct to handle Requests
///
#[derive(Clone)]
pub struct Request {
    /// The requested Uri
    pub uri: UrlEncodeData,
    /// All headers of the request that haven't been parsed
    pub headers: HeaderMap,
    /// The methods Request represented with the [Method]
    pub method: Method,
    /// An optional HashMap representation of all Cookies of the request
    pub cookies: Option<HashMap<String, String>>,
    /// If the has a body it represents the [Mime]-type of the body
    pub mime_type: Option<(Mime, String)>,
    // pub connection: ConnectionMeta,
}

// struct ConnectionMeta {
//     remote: Option<SocketAddr>,
//     // certificates
// }

#[derive(Clone, Debug, Copy, PartialEq, Eq, PartialOrd, Ord)]
/// Media Types in which a Route can be requested to ansewr, optional for routes
pub enum MediaType {
    /// Json Data
    Json,
    /// Plain Text
    Plain,
    /// HTML Text
    Html,
}

#[derive(Debug, PartialEq, Eq, PartialOrd, Ord)]
pub enum ParseErrors {
    NoData,
    BadData,
}

#[derive(Debug, PartialEq, Eq, PartialOrd, Ord)]
/// Errors that Occur when a Form can't be parsed
pub struct ParseFormError {
    pub error: ParseErrors,
}

impl Display for ParseFormError {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        write!(f, "{}", self.error)
    }
}

impl Display for ParseErrors {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        match self {
            ParseErrors::NoData => write!(f, "No Data at key"),
            ParseErrors::BadData => write!(f, "Bad Data at key"),
        }
    }
}

impl Error for ParseFormError {}
